websocket

package
v0.4.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Jan 6, 2026 License: MIT Imports: 2 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func New

func New(handler Handler) amaro.Handler

New creates a new Amaro handler that upgrades the connection to a WebSocket. It wraps golang.org/x/net/websocket.

Types

type Handler

type Handler func(*websocket.Conn)

Handler is a type alias for the websocket handler function.

Jump to